"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/** @jsx jsx */
var core_1 = require("@emotion/core");
var React = tslib_1.__importStar(require("react"));
var Text_1 = require("./Text");
var visually_hidden_1 = tslib_1.__importDefault(require("@reach/visually-hidden"));
var prop_types_1 = tslib_1.__importDefault(require("prop-types"));
var colors_1 = require("./Theme/colors");
var use_uid_1 = require("./Hooks/use-uid");
var Providers_1 = require("./Theme/Providers");
var Button_1 = require("./Button");
var Icons_1 = require("./Icons");
var compose_bind_1 = require("./Hooks/compose-bind");
var use_media_1 = require("use-media");
var getInputSizes = function (theme) { return ({
    sm: core_1.css({
        fontSize: theme.fontSizes[0],
        padding: "0.25rem 0.5rem"
    }),
    md: core_1.css({
        fontSize: theme.fontSizes[1],
        padding: "0.5rem 0.75rem"
    }),
    lg: core_1.css({
        fontSize: theme.fontSizes[2],
        padding: "0.65rem 1rem"
    })
}); };
var InputGroupContext = React.createContext({
    uid: undefined,
    error: undefined
});
exports.InputGroup = function (_a) {
    var id = _a.id, label = _a.label, children = _a.children, error = _a.error, helpText = _a.helpText, hideLabel = _a.hideLabel, labelStyle = _a.labelStyle, labelTextStyle = _a.labelTextStyle, other = tslib_1.__rest(_a, ["id", "label", "children", "error", "helpText", "hideLabel", "labelStyle", "labelTextStyle"]);
    var uid = use_uid_1.useUid(id);
    var theme = Providers_1.useTheme();
    var isDark = theme.colors.mode === "dark";
    var danger = isDark
        ? theme.colors.intent.danger.light
        : theme.colors.intent.danger.base;
    return (core_1.jsx("div", tslib_1.__assign({ className: "InputGroup", css: {
            marginTop: theme.spaces.md,
            ":first-child": {
                marginTop: 0
            }
        } }, other),
        core_1.jsx(exports.Label, { hide: hideLabel, htmlFor: uid, css: labelStyle, textStyle: labelTextStyle }, label),
        core_1.jsx(InputGroupContext.Provider, { value: {
                uid: uid,
                error: error
            } }, children),
        error && typeof error === "string" ? (core_1.jsx("div", { className: "InputGroup__error", css: {
                alignItems: "center",
                marginTop: theme.spaces.sm,
                display: "flex"
            } },
            core_1.jsx(Icons_1.IconAlertCircle, { size: "sm", color: danger }),
            core_1.jsx(Text_1.Text, { css: {
                    display: "block",
                    marginLeft: theme.spaces.xs,
                    fontSize: theme.fontSizes[0],
                    color: danger
                } }, error))) : (error),
        helpText && (core_1.jsx(Text_1.Text, { className: "InputGroup__help", css: {
                display: "block",
                marginTop: theme.spaces.xs,
                color: theme.colors.text.muted,
                fontSize: theme.fontSizes[0]
            }, variant: "body" }, helpText))));
};
exports.InputGroup.propTypes = {
    label: prop_types_1.default.string.isRequired,
    hideLabel: prop_types_1.default.bool,
    helpText: prop_types_1.default.string,
    error: prop_types_1.default.oneOfType([prop_types_1.default.string, prop_types_1.default.node]),
    id: prop_types_1.default.string,
    children: prop_types_1.default.node
};
function shadowBorder(color, opacity) {
    return "0 0 0 2px transparent inset, 0 0 0 1px " + colors_1.alpha(color, opacity) + " inset";
}
function getBaseStyles(theme) {
    var dark = theme.colors.mode === "dark";
    var baseStyles = core_1.css({
        display: "block",
        width: "100%",
        lineHeight: theme.lineHeights.body,
        color: theme.colors.text.default,
        backgroundColor: "transparent",
        backgroundImage: "none",
        backgroundClip: "padding-box",
        WebkitFontSmoothing: "antialiased",
        WebkitTapHighlightColor: "transparent",
        WebkitAppearance: "none",
        boxSizing: "border-box",
        touchAction: "manipulation",
        fontFamily: theme.fonts.base,
        border: "none",
        boxShadow: dark
            ? shadowBorder(theme.colors.palette.gray.lightest, 0.14)
            : shadowBorder(theme.colors.palette.gray.dark, 0.2),
        borderRadius: theme.radii.sm,
        transition: "background 0.25s cubic-bezier(0.35,0,0.25,1), border-color 0.15s cubic-bezier(0.35,0,0.25,1), box-shadow 0.15s cubic-bezier(0.35,0,0.25,1)",
        "::placeholder": {
            color: colors_1.alpha(theme.colors.text.default, 0.45)
        },
        ":focus": {
            boxShadow: dark
                ? Button_1.focusShadow(colors_1.alpha(theme.colors.palette.blue.light, 0.5), colors_1.alpha(theme.colors.palette.gray.dark, 0.4), colors_1.alpha(theme.colors.palette.gray.light, 0.2))
                : Button_1.focusShadow(colors_1.alpha(theme.colors.palette.blue.dark, 0.1), colors_1.alpha(theme.colors.palette.gray.dark, 0.2), colors_1.alpha(theme.colors.palette.gray.dark, 0.05)),
            outline: "none"
        },
        ":disabled": {
            opacity: dark ? 0.4 : 0.8,
            background: theme.colors.background.tint1,
            cursor: "not-allowed",
            boxShadow: dark
                ? shadowBorder(theme.colors.palette.gray.lightest, 0.15)
                : shadowBorder(theme.colors.palette.gray.dark, 0.12)
        },
        ":active": {
            background: theme.colors.background.tint1
        }
    });
    return baseStyles;
}
function useActiveStyle() {
    var _a = React.useState(false), active = _a[0], setActive = _a[1];
    return {
        bind: {
            onTouchStart: function () { return setActive(true); },
            onTouchEnd: function () { return setActive(false); }
        },
        active: active
    };
}
function useSharedStyle() {
    var theme = Providers_1.useTheme();
    var errorStyles = {
        boxShadow: shadowBorder(theme.colors.intent.danger.base, 0.45)
    };
    var baseStyles = React.useMemo(function () { return getBaseStyles(theme); }, [theme]);
    var inputSizes = React.useMemo(function () { return getInputSizes(theme); }, [theme]);
    var activeBackground = core_1.css({ background: theme.colors.background.tint1 });
    return {
        baseStyles: baseStyles,
        inputSizes: inputSizes,
        activeBackground: activeBackground,
        errorStyles: errorStyles
    };
}
/**
 * Our basic Input element. Use this when building customized
 * forms. Otherwise, stick with InputGroup
 */
exports.InputBase = React.forwardRef(function (_a, ref) {
    var autoComplete = _a.autoComplete, autoFocus = _a.autoFocus, _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["autoComplete", "autoFocus", "inputSize", "topDivStyle"]);
    var _c = React.useContext(InputGroupContext), uid = _c.uid, error = _c.error;
    var _d = useActiveStyle(), bind = _d.bind, active = _d.active;
    var _e = useSharedStyle(), baseStyles = _e.baseStyles, inputSizes = _e.inputSizes, activeBackground = _e.activeBackground, errorStyles = _e.errorStyles;
    var height = Button_1.getHeight(inputSize);
    return (core_1.jsx("input", tslib_1.__assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            active && activeBackground,
            error && errorStyles,
            { height: height },
            topDivStyle
        ] }, compose_bind_1.safeBind({ ref: ref }, other))));
});
exports.InputBase.propTypes = {
    inputSize: prop_types_1.default.oneOf(["sm", "md", "lg"]),
    autoComplete: prop_types_1.default.bool,
    autoFocus: prop_types_1.default.bool
};
//包裹一个div以便于控制宽度和对齐。
exports.Input = React.forwardRef(function (_a, ref) {
    var autoComplete = _a.autoComplete, autoFocus = _a.autoFocus, _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, _c = _a.fullWidth, fullWidth = _c === void 0 ? true : _c, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["autoComplete", "autoFocus", "inputSize", "fullWidth", "topDivStyle"]);
    var _d = React.useContext(InputGroupContext), uid = _d.uid, error = _d.error;
    var _e = useActiveStyle(), bind = _e.bind, active = _e.active;
    var _f = useSharedStyle(), baseStyles = _f.baseStyles, inputSizes = _f.inputSizes, activeBackground = _f.activeBackground, errorStyles = _f.errorStyles;
    var height = Button_1.getHeight(inputSize);
    return (core_1.jsx("div", { css: [
            {
                textAlign: 'left',
                width: "100%"
            },
            topDivStyle
        ] },
        core_1.jsx("input", tslib_1.__assign({ id: uid, className: "Input", autoComplete: autoComplete, autoFocus: autoFocus }, bind, { css: [
                baseStyles,
                inputSizes[inputSize],
                active && activeBackground,
                error && errorStyles,
                { height: height },
                !fullWidth && {
                    width: 'unset',
                }
            ] }, compose_bind_1.safeBind({ ref: ref }, other)))));
});
/**
 * Textarea version of InputBase
 */
exports.TextArea = function (_a) {
    var _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["inputSize", "topDivStyle"]);
    var _c = useActiveStyle(), bind = _c.bind, active = _c.active;
    var _d = useSharedStyle(), baseStyles = _d.baseStyles, inputSizes = _d.inputSizes, activeBackground = _d.activeBackground, errorStyles = _d.errorStyles;
    var _e = React.useContext(InputGroupContext), uid = _e.uid, error = _e.error;
    return (core_1.jsx("textarea", tslib_1.__assign({ className: "TextArea", id: uid }, bind, { css: [
            baseStyles,
            inputSizes[inputSize],
            {
                overflow: "auto",
                resize: "vertical",
            },
            active && activeBackground,
            error && errorStyles,
            topDivStyle
        ] }, other)));
};
exports.TextArea.propTypes = {
    inputSize: prop_types_1.default.oneOf(["sm", "md", "lg"])
};
/**
 * A styled Label to go along with input elements
 */
exports.Label = function (_a) {
    var children = _a.children, hide = _a.hide, textStyle = _a.textStyle, other = tslib_1.__rest(_a, ["children", "hide", "textStyle"]);
    var theme = Providers_1.useTheme();
    var child = (core_1.jsx("label", tslib_1.__assign({ className: "Label", css: {
            display: "inline-block",
            marginBottom: hide ? 0 : theme.spaces.sm
        } }, other),
        core_1.jsx(Text_1.Text, { className: "Label__text", variant: "subtitle", css: textStyle }, children)));
    return hide ? core_1.jsx(visually_hidden_1.default, null, child) : child;
};
exports.Label.propTypes = {
    hide: prop_types_1.default.bool,
    children: prop_types_1.default.node
};
/**
 * A styled select menu
 */
exports.Select = function (_a) {
    var multiple = _a.multiple, _b = _a.inputSize, inputSize = _b === void 0 ? "md" : _b, divStyle = _a.divStyle, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["multiple", "inputSize", "divStyle", "topDivStyle"]);
    var theme = Providers_1.useTheme();
    var inputSizes = getInputSizes(theme);
    var _c = React.useContext(InputGroupContext), uid = _c.uid, error = _c.error;
    var selectSize = {
        sm: inputSizes.sm,
        md: inputSizes.md,
        lg: inputSizes.lg
    };
    var dark = theme.colors.mode === "dark";
    var height = Button_1.getHeight(inputSize);
    return (core_1.jsx("div", { css: [
            {
                textAlign: 'left'
            },
            topDivStyle
        ] },
        core_1.jsx("div", { className: "Select", css: [
                {
                    position: "relative",
                    display: 'inline-flex',
                    width: '100%'
                },
                divStyle,
            ] },
            core_1.jsx("select", tslib_1.__assign({ className: "Select__input", id: uid, css: [
                    selectSize[inputSize],
                    {
                        WebkitAppearance: "none",
                        display: "block",
                        width: "100%",
                        lineHeight: theme.lineHeights.body,
                        height: height,
                        color: theme.colors.text.default,
                        background: "transparent",
                        fontFamily: theme.fonts.base,
                        boxShadow: "0 0 0 2px transparent inset, 0 0 0 1px " + (dark
                            ? colors_1.alpha(theme.colors.palette.gray.lightest, 0.14)
                            : colors_1.alpha(theme.colors.palette.gray.dark, 0.2)) + " inset",
                        border: "none",
                        backgroundClip: "padding-box",
                        borderRadius: theme.radii.sm,
                        margin: 0,
                        ":disabled": {
                            ":disabled": {
                                opacity: dark ? 0.4 : 0.8,
                                background: theme.colors.background.tint1,
                                cursor: "not-allowed",
                                boxShadow: "0 0 0 2px transparent inset, 0 0 0 1px " + (dark
                                    ? colors_1.alpha(theme.colors.palette.gray.lightest, 0.15)
                                    : colors_1.alpha(theme.colors.palette.gray.dark, 0.12)) + " inset"
                            }
                        },
                        ":focus": {
                            borderColor: theme.colors.palette.blue.base,
                            boxShadow: dark
                                ? Button_1.focusShadow(colors_1.alpha(theme.colors.palette.blue.light, 0.5), colors_1.alpha(theme.colors.palette.gray.dark, 0.4), colors_1.alpha(theme.colors.palette.gray.light, 0.2))
                                : Button_1.focusShadow(colors_1.alpha(theme.colors.palette.blue.dark, 0.1), colors_1.alpha(theme.colors.palette.gray.dark, 0.2), colors_1.alpha(theme.colors.palette.gray.dark, 0.05)),
                            outline: 0
                        }
                    },
                    error && {
                        boxShadow: shadowBorder(theme.colors.intent.danger.base, 0.45)
                    }
                ], multiple: multiple }, other)),
            !multiple && (core_1.jsx(Icons_1.IconChevronDown, { className: "Select__icon", color: theme.colors.text.muted, css: {
                    position: "absolute",
                    top: "50%",
                    right: "0.75rem",
                    transform: "translateY(-50%)",
                    pointerEvents: "none"
                } })))));
};
exports.Select.propTypes = {
    inputSize: prop_types_1.default.oneOf(["sm", "md", "lg"]),
    multiple: prop_types_1.default.bool
};
exports.Check = function (_a) {
    var label = _a.label, id = _a.id, disabled = _a.disabled, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["label", "id", "disabled", "topDivStyle"]);
    var uid = use_uid_1.useUid(id);
    var theme = Providers_1.useTheme();
    return (core_1.jsx("div", tslib_1.__assign({ className: "Check", css: [
            {
                textAlign: 'left',
                display: "inline-flex",
                alignItems: "center"
            },
            topDivStyle
        ] }, other),
        core_1.jsx("input", tslib_1.__assign({ disabled: disabled, className: "Check__input", type: "checkbox", id: uid, css: [
                {
                    height: '2rem',
                    width: '2rem',
                    display: 'inline-flex',
                },
            ] }, other)),
        core_1.jsx("label", { className: "Check__label", css: {
                opacity: disabled ? 0.6 : undefined,
                marginLeft: theme.spaces.xs
            }, htmlFor: uid },
            core_1.jsx(Text_1.Text, null, label))));
};
exports.Check.propTypes = {
    label: prop_types_1.default.string.isRequired,
    id: prop_types_1.default.string,
    disabled: prop_types_1.default.bool
};
/**
 * A styled Label to go along with input elements
 * 若children有，就是有附带单位后缀串的模式；
 * 带单位后缀的说明，70%给输入框，后面30%给叙述单位字串{空格也算}
 */
//输入的 单位说明 字符串 放在 <Text className="Suffix__text"  >
exports.SuffixInput = function (_a) {
    var children = _a.children, hide = _a.hide, textStyle = _a.textStyle, inputSize = _a.inputSize, topDivStyle = _a.topDivStyle, other = tslib_1.__rest(_a, ["children", "hide", "textStyle", "inputSize", "topDivStyle"]);
    var theme = Providers_1.useTheme();
    return (core_1.jsx("div", { css: [
            {
                textAlign: 'left'
                //display: "inline-block",
            },
            topDivStyle
        ] },
        core_1.jsx(exports.InputBase, tslib_1.__assign({ inputSize: inputSize, css: {
                display: "inline-block",
                width: children ? "70%" : '100%',
            } }, other)),
        core_1.jsx(Text_1.Text, { className: "Suffix__text", variant: "subtitle", css: [
                {
                    display: children ? "inline-flex" : 'none',
                    paddingLeft: '0.2rem'
                },
                textStyle
            ] }, children)));
};
exports.SuffixInput.propTypes = {
    hide: prop_types_1.default.bool,
    children: prop_types_1.default.node
};
//修改InputGroup排版模式; 并排模式，根据屏幕自适应。支持 2 个模式的布局安排结构。
exports.InputGroupLine = function (_a) {
    var id = _a.id, label = _a.label, children = _a.children, error = _a.error, helpText = _a.helpText, hideLabel = _a.hideLabel, labelStyle = _a.labelStyle, labelTextStyle = _a.labelTextStyle, lineStyle = _a.lineStyle, _b = _a.switchPx, switchPx = _b === void 0 ? 360 : _b, other = tslib_1.__rest(_a, ["id", "label", "children", "error", "helpText", "hideLabel", "labelStyle", "labelTextStyle", "lineStyle", "switchPx"]);
    var uid = use_uid_1.useUid(id);
    var theme = Providers_1.useTheme();
    var isDark = theme.colors.mode === "dark";
    var danger = isDark
        ? theme.colors.intent.danger.light
        : theme.colors.intent.danger.base;
    //根据外部程序制定的px数，来决定用哪一个模式布局。
    var fitable = use_media_1.useMedia({ minWidth: switchPx + "px" });
    var labelDivCss = core_1.css({
        flex: 1,
        paddingRight: '0.8rem'
    }, labelTextStyle);
    var childNodeVar = (core_1.jsx(InputGroupContext.Provider, { value: {
            uid: uid,
            error: error
        } }, React.cloneElement(children, {
        topDivStyle: { flex: '1 1 60%' },
    })));
    return (core_1.jsx("div", tslib_1.__assign({ className: "InputGroup", css: {
            marginTop: theme.spaces.md,
            ":first-child": {
                marginTop: 0
            },
            textAlign: 'center'
        } }, other),
        core_1.jsx("div", { css: [
                {
                    alignItems: "center",
                    justifyContent: "space-around",
                    display: "flex",
                    flexWrap: 'wrap',
                    maxWidth: '950px',
                    margin: '0 auto',
                    paddingRight: fitable ? '0.5rem' : 'unset',
                },
                lineStyle
            ] },
            core_1.jsx(exports.Label, { hide: hideLabel, htmlFor: uid, textStyle: labelDivCss, css: [
                    {
                        display: "inline-flex",
                        textAlign: fitable ? "right" : "left",
                        flex: '1 1 40%',
                    },
                    labelStyle
                ] }, label),
            fitable && childNodeVar),
        !fitable && childNodeVar,
        error && typeof error === "string" ? (core_1.jsx("div", { className: "InputGroup__error", css: {
                alignItems: "center",
                marginTop: theme.spaces.sm,
                display: "flex",
                justifyContent: 'center'
            } },
            core_1.jsx(Icons_1.IconAlertCircle, { size: "sm", color: danger }),
            core_1.jsx(Text_1.Text, { css: {
                    display: "block",
                    marginLeft: theme.spaces.xs,
                    fontSize: theme.fontSizes[0],
                    color: danger
                } }, error))) : (error),
        helpText && (core_1.jsx(Text_1.Text, { className: "InputGroup__help", css: {
                display: "inline-flex",
                marginTop: theme.spaces.xs,
                color: theme.colors.text.muted,
                fontSize: theme.fontSizes[0]
            }, variant: "body" }, helpText))));
};
exports.InputGroupLine.propTypes = {
    label: prop_types_1.default.string.isRequired,
    hideLabel: prop_types_1.default.bool,
    helpText: prop_types_1.default.string,
    error: prop_types_1.default.oneOfType([prop_types_1.default.string, prop_types_1.default.node]),
    id: prop_types_1.default.string,
    children: prop_types_1.default.node
};
